/////////////////////////////////////////////////////////////////
// Program to demonstrate usage of the MRST 2006 NNLO PDFs.    //
// to calculate errors.                                        //
/////////////////////////////////////////////////////////////////
#include "LHAPDF/LHAPDF.h"
#include <iostream>
#include <cmath>
#include <cstdio>
#include <cstdlib>
using namespace std;

using namespace LHAPDF;


int main() {
  // Show initialisation banner only once
  setVerbosity(LOWKEY); // or SILENT, for no banner at all

  // You could explicitly set the path to the PDFsets directory
  // setPDFPath("/home/whalley/local/share/lhapdf/PDFsets");
  
  // Initialize PDF sets
  const string NAME = "MRST2006nnlo";
  // const string NAME = "ZEUS2005_ZJ";
  initPDFSetM(1, NAME, LHGRID);
  initPDFSetM(2, NAME, LHGRID);
  initPDFSetM(3, NAME, LHGRID);
  
  // Find the number of eigensets from numberPDF()
  int neigen = numberPDFM(1)/2;
  cout << "Number of eigensets in this fit = " << neigen << endl;
  // Find the min and max values of x and q2 
  double xmin = getXmin(0); 
  double xmax = getXmax(0);
  //double q2min = getQ2min(0);
  //double q2max = getQ2max(0);
  
  // Number of x values
  const int nx = 10;
  vector<double> fc(10), summax(10), summin(10), sum(10);
  double fp, fm;
  
  // Set the q scale
  double q = 10;
  int flav = 4;
  // loop over all flavours from -5 to +5
  
  // Find central value
  for (int ieigen = 1; ieigen <= neigen; ++ieigen) {
    for (int ix = 0; ix < nx; ++ix){
      const double x = pow(10.0, log10(xmin) + (ix-1.)/(nx-1.)*(log10(xmax)-log10(xmin)));
     
      // Find central value
      // NB. This is very inefficient, because in low memory mode the
      // sets need to be re-initialised with every loop iteration. 
      // Be careful if you try to take the initialisations out of the loop!
      initPDFM(1, 0);
      fc[ix] = xfxM(1,x,q,flav);
      
      // Loop over eigensets 
      summax[ix] = 0;
      summin[ix] = 0;
      sum[ix] = 0;
      // NB. Same inefficient re-initialisation as above
      initPDFM(2, 2*ieigen-1);
      fp = xfxM(2,x,q,flav);
      // NB. Same inefficient re-initialisation as above
      initPDFM(3, 2*ieigen);
      fm = xfxM(3,x,q,flav);
      summax[ix] += pow(max(max(fp-fc[ix],fm-fc[ix]),0.),2);
      summin[ix] += pow(max(max(fc[ix]-fp,fc[ix]-fm),0.),2);
      sum[ix] += pow(fp-fm,2);
    }
  }
  cout << "flavour = " << flav << "               Asymmetric (%)   Symmetric (%)" << endl;
  cout << "     x    Q**2    xf(x)     plus       minus      +-      " << endl;
  for  (int ix = 0; ix < nx; ++ix) {
    const double x = pow(10.0, log10(xmin) + (ix-1.)/(nx-1.)*(log10(xmax)-log10(xmin)));
    printf("%0.7f %.0f %10.2E %+8.2f %+8.2f %8.2f \n",
           x, q*q, fc[ix], 
           sqrt(summax[ix])*100/fc[ix],
           -sqrt(summin[ix])*100/fc[ix],
           0.5*sqrt(sum[ix])*100/fc[ix]);
  }
  
  return EXIT_SUCCESS;
}


#include "FortranWrappers.h"
#ifdef FC_DUMMY_MAIN
int FC_DUMMY_MAIN() { return 1; }
#endif
